Add-Type -AssemblyName "System.DirectoryServices.Protocols"

function SendLDAPRequest {
    [CmdletBinding()]
    [OutputType([System.DirectoryServices.Protocols.DirectoryResponse])]
    param()

    $directory = New-Object System.DirectoryServices.Protocols.LdapDirectoryIdentifier -ArgumentList($directoryServer,$false,$false)
    $ldapConnection =  New-Object System.DirectoryServices.Protocols.LdapConnection -ArgumentList($directory, $lookupUser)
    
    if($lookupUsername -ne $null -and $lookupUsername -ne "")
    {
        $ldapConnection.Credential = New-Object System.Net.NetworkCredential -ArgumentList($lookupUsername, $lookupUserPassword)
        $ldapConnection.AuthType = [System.DirectoryServices.Protocols.AuthType]::Basic
    }

    $ldapConnection.SessionOptions.ProtocolVersion = 3
    $ldapConnection.SessionOptions.SecureSocketLayer = $directoryServerUseTLS

    $request = New-Object System.DirectoryServices.Protocols.SearchRequest($baseDN, $aquaUsersQuery, [System.DirectoryServices.Protocols.SearchScope]::Subtree);    

    return $ldapConnection.SendRequest($request);   
}

function GetDirectoryUsers {
    [CmdletBinding()]
    [OutputType([array])]
    param()

    $result = @()
    
    $response = SendLDAPRequest
    
    foreach($entry in $response.Entries)
    {
        $disabled = IsUserDisabled($entry)
        
        $user = @{}
        $user.DistinguishedName = $entry.DistinguishedName
        
        if($entry.Attributes[$aquaLoginAttribute] -ne $null) {
            $user.Username = $entry.Attributes[$aquaLoginAttribute][0]
        }
        if($entry.Attributes[$aquaFirstNameAttribute] -ne $null) {
            $user.FirstName = $entry.Attributes[$aquaFirstNameAttribute][0]
        }
        if($entry.Attributes[$aquaLastNameAttribute] -ne $null) {
            $user.LastName = $entry.Attributes[$aquaLastNameAttribute][0]
        }
        if($entry.Attributes[$aquaEmailAttribute] -ne $null) {
            $user.Email = $entry.Attributes[$aquaEmailAttribute][0]
        }
        
        $user.Disabled = $disabled
        $result += $user
    }
    
    return $result
}


function IsUserDisabled {
    [CmdletBinding()]
    [OutputType([bool])]
    Param (
        [Parameter(Mandatory=$true)]
        [System.DirectoryServices.Protocols.SearchResultEntry]
        $entry
    )

    # For Active Directory: 
    # The userAccountControl attribute contains flags regarding the account status
    # Bit 1 indicates whether the account is disabled or not
    #$useraccountcontrol = $entry.Attributes["useraccountcontrol"][0]
    #return ($useraccountcontrol -band 2) -eq 2
    return $false
}