# This file performs a READ-ONLY test (retrieval) of data from LDAP service, 
# same as done in UserSync.ps1 script. 
# This script makes NO modifications neither in the directory nor in aqua. 
[CmdletBinding()]
param (
    # Indicates that the raw LDAP entries should be dumped as well.
    [Parameter()]
    [switch]
    $DumpEntries
)

Import-Module "${PSScriptRoot}\DirectoryFunctionsLDAP.psm1" -Force
Import-Module "${PSScriptRoot}.\Logging.psm1" -Force

$global:logfile = "${PSScriptRoot}\logs\Test.log"

. "${PSScriptRoot}\Configuration.ps1"

try {
    if ($DumpEntries) {
        $response = SendLDAPRequest
        if ($response.Entries.Count -eq 0) {
            LogInfo "No matching LDAP entries found"
        } else {
            $message = "Found LDAP entries:`n"
            foreach($entry in $response.Entries) {
                $message += "Entry $($entry.DistinguishedName):`n"
                foreach($key in $entry.Attributes.Keys) {
                    $message += "`t${key}: $($entry.Attributes[$key][0])`n"
                }
            }
            LogInfo $message
        }
    }

    $users = GetDirectoryUsers
    if ($users.Count -eq 0) {
        LogInfo "No matching users found"
    } else {
        $message = "Extracted user data:`n"
        foreach ($user in $users) {
            $message += "User $($user.DistinguishedName):`n"
            foreach ($key in $user.Keys) {
                $message += "`t${key}: $($user[$key])`n"
            }
        }
        LogInfo $message
    }

    Write-Output "Test completed. Check log file for results."
}
catch [Exception] {
    LogError $_.Exception
    Write-Output ($_.Exception | Format-List -Force)
}